import requests
from typing import List, Optional
from datetime import datetime
from pydantic import BaseModel, Field
from datetime import datetime


class NewsItem(BaseModel):
    by: str
    id: int
    score: int
    time: int
    title: str
    type: str
    url: Optional[str] = None
    kids: Optional[List[int]] = Field(default=None)
    text: Optional[str] = None
    descendants: int = 0
    deleted: bool = Field(default=False)
    dead: str = Field(default=False)
    poll: Optional[int] = None
    parent: Optional[int] = None
    parts: Optional[list[int]] = None

    def is_recent(self):
        return self.age() < 24

    def news_url(self):
        return f"https://news.ycombinator.com/item?id={self.id}"

    def age(self):
        current_time = datetime.now()
        timestamp_datetime = datetime.fromtimestamp(self.time)
        return abs((current_time - timestamp_datetime).total_seconds() / 3600)

    def diff_for_humans(self):
        timestamp_datetime = datetime.fromtimestamp(self.time)
        return timesince(timestamp_datetime)

    def __str__(self):
        return f"{self.title} ({self.descendants})"

    def __repr__(self):
        return f"NewsItem({self.title} || {self.descendants} comments | {self.diff_for_humans()})"


def recent_top_stories() -> List[NewsItem]:
    url = "https://hacker-news.firebaseio.com/v0/topstories.json"
    headers = {
        "Accept": "application/json",
        "User-Agent": "(botmatt, contact@publicmatt.com)",
    }
    response = requests.get(url, headers=headers)
    match response.status_code:
        case 200:
            item_ids = response.json()
        case _:
            return []

    items = []
    for item_id in item_ids[:100]:
        url = f"https://hacker-news.firebaseio.com/v0/item/{item_id}.json"
        response = requests.get(url, headers=headers)
        match response.status_code:
            case 200:
                item = response.json()
            case _:
                continue
        item = NewsItem(**item)
        items.append(item)

    recent = list(filter(lambda d: d.is_recent(), items))
    recent.sort(key=lambda x: x.descendants, reverse=True)
    return recent[:10]


def timesince(dt, default="now"):
    now = datetime.now()
    diff = now - dt
    periods = (
        (diff.days / 365, "year", "years"),
        (diff.days / 30, "month", "months"),
        (diff.days / 7, "week", "weeks"),
        (diff.days, "day", "days"),
        (diff.seconds / 3600, "hour", "hours"),
        (diff.seconds / 60, "minute", "minutes"),
        (diff.seconds, "second", "seconds"),
    )
    for period, singular, plural in periods:
        if period >= 1:
            return "%d %s ago" % (period, singular if period == 1 else plural)
    return default
